/*
 * logger.plugin - A regular expression logger for Gnome-Mud
 * Copyright (C) 2004  Quim Rovira
 * Created on 22-07-2004
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#define __MODULE__
#include <gtk/gtk.h>
#include <glib.h>
#include <glade/glade.h>
#include <pcre.h>
#include <string.h>
#include <vte/vte.h>
#include "../../src/modules_api.h"
#include "main.h"

PLUGIN_INFO gnomemud_plugin_info =
{
	"Logger Plugin",
	"Quim Rovira",
	"0.1",
	"Simple logger plugin that\n uses regular expresions\n to match input.",
	init_plugin,
};

LOGGER_CONFIG logger_config;
GladeXML *gladexml;




void load_defaults() {
	GSList *sl;
	
	logger_config.enabled = FALSE;
	logger_config.autosave_interval = -1; //Disabled
	//Free first if not empty
	if(logger_config.regexps != NULL) {
		sl = logger_config.regexps;
		while(sl!=NULL) {
			if(sl->data != NULL) {
				if(((LOGGER_REGEXP *)sl->data)->regexp != NULL)
					g_free(((LOGGER_REGEXP *)sl->data)->regexp);
				if(((LOGGER_REGEXP *)sl->data)->target_window != NULL)
					g_free(((LOGGER_REGEXP *)sl->data)->target_window);
				if(((LOGGER_REGEXP *)sl->data)->color != NULL)
					g_free(((LOGGER_REGEXP *)sl->data)->color);
				g_free((LOGGER_REGEXP *)sl->data);
			}
			sl = g_slist_next(sl);
		}
		g_slist_free(logger_config.regexps);
	}
	logger_config.regexps = NULL;
}

gchar* lookup_conf_file(gchar *fname) {
	gchar *fullpath;
	
	fullpath = g_strconcat(g_get_home_dir(),"/.gnome-mud/",fname,NULL);
	if(g_file_test(fullpath, G_FILE_TEST_EXISTS)) {
		return fullpath;
	}
	fullpath = g_strconcat(PREFIX,"/share/gnome-mud/",fname,NULL);
//	g_message("Testing... \"%s\"",
	if(g_file_test(fullpath, G_FILE_TEST_EXISTS)) {
		return fullpath;
	}
	fullpath = g_strconcat("/usr/share/gnome-mud/",fname,NULL);
	if(g_file_test(fullpath, G_FILE_TEST_EXISTS)) {
		return fullpath;
	}
	return NULL;
}

VteTerminal *add_term_page(gchar *name) {
	GtkNotebook *note;
	GtkLabel *label;
	VteTerminal *vte;
	GtkWidget *hbox;
	GtkWidget *vbar;
	
	if(gladexml == NULL) {
		g_warning("Interface not loaded!");
		return NULL;
	}
	note = (GtkNotebook*) glade_xml_get_widget(gladexml,"vtenotebook");
	if(note == NULL) {
		g_warning("Error: GtkNotebook not found!");
		return NULL;
	}
	
	hbox = gtk_hbox_new(FALSE,0);
	label = (GtkLabel*) gtk_label_new(name);
	vte = (VteTerminal*) vte_terminal_new();
	vbar = gtk_vscrollbar_new(NULL);
	gtk_range_set_adjustment(GTK_RANGE(vbar), VTE_TERMINAL(vte)->adjustment);

	gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(vte), TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), vbar, FALSE, FALSE, 0);
	
	gtk_notebook_append_page(note,hbox,GTK_WIDGET(label));
	if(vte != NULL) {
//		g_message("vte for %s created!",name);
	}
	gtk_widget_show_all(GTK_WIDGET(note));
	return vte;
}

VteTerminal *get_terminal(gchar *name) {
	gint i;
	GtkNotebook *note;
	
	note = (GtkNotebook*)glade_xml_get_widget(gladexml,"vtenotebook");
	if(note == NULL) {
		g_warning("Error: Could not find GtkNotebook");
		return NULL;
	}
	
	for(i=0;i<gtk_notebook_get_n_pages(note);i++) {
		GtkWidget *hbox;
		GtkLabel *label;
		
//		vte = VTE_TERMINAL(gtk_notebook_get_nth_page(note,i));
		hbox = gtk_notebook_get_nth_page(note,i);
		label = GTK_LABEL(gtk_notebook_get_tab_label(note,hbox));
		if(strcmp(name,gtk_label_get_text(label))==0) {
			GList *gl;
			VteTerminal *vte;
			
			gl = gtk_container_get_children(GTK_CONTAINER(hbox));
			vte = VTE_TERMINAL(g_list_nth_data(gl,0));
			g_list_free(gl);
			
//			g_message("Found terminal %s!",name);
			return vte;
		}
	}
	return NULL;
}


void parse_config() {
	gchar *str, prop[512], val[512];
	FILE *fp;
	
	logger_config.enabled = FALSE;
	logger_config.autosave_interval = -1; //Disabled
	logger_config.regexps = NULL;
	
	str = lookup_conf_file("logger.conf");
	if(str==NULL) {
		g_message("No configuration file found!");
		return;
	}

//	g_message("Configuration file \"%s\" found.",str);
	fp=fopen(str,"r");
	g_free(str);
	if(fp==NULL) {
		g_message("Logger plugin: No logger.conf file found!");
		return;
	}
	str=g_new(gchar,1024);
	while(!feof(fp)) {
		if(fgets(str,1024*sizeof(gchar),fp)==NULL) {
			//Read error
			//g_message("Logger plugin: Read error! \"%s\"",str);
			break;
		}
		g_strchug(str); // Do not remove trailing whitespace
//		g_message("Parsed : \"%s\"\n",str);
		if(strlen(str)==0) // Skip empty lines
			continue;
		if(*str=='#') // Skip comments
			continue;
		if(sscanf(str,"%512[^=\n]=%512[^\n]\n",(char *)&prop,(char *)&val)==2) {
			if(strcmp(prop,"enabled")==0) {
				logger_config.enabled = !(strcmp("no",val)==0);
			} else if(strcmp(prop,"autosave_interval")==0) {
				if(sscanf(val,"%d",&logger_config.autosave_interval)==0)
					logger_config.autosave_interval=-1;
			} else if(strcmp(prop,"regexp")==0) {
				LOGGER_REGEXP *lre;
				
				lre = g_new(LOGGER_REGEXP, 1);
				lre->regexp = g_new(gchar,512);
				lre->target_window = g_new(gchar,512);
				lre->color = g_new(gchar,512);
				sscanf(val,"%511[^;];%511[^;];%511[^;];",
							lre->target_window, lre->color, lre->regexp);
				logger_config.regexps = g_slist_append(logger_config.regexps,lre);
//				g_message("Got regexp /%s/ (window %s, color %s) from conf file.",
//							lre->regexp, lre->target_window, lre->color);
			} else  {
				g_message("Logger plugin: Bogus conf line \"%s\"!",str);
			}
		} else {
			g_message("Logger plugin: Bogus conf line \"%s\"!",str);
		}
		
	}
	fclose(fp);
}

void study_re(gpointer plre, gpointer user_data) {
	LOGGER_REGEXP *lre = (LOGGER_REGEXP *) plre;
	const gchar *error;
	gint error_offset;
	
	lre->re = pcre_compile(lre->regexp,0,&error,&error_offset,NULL);
	if(lre->re == NULL) {
		g_warning("Pattern /%s/ did not compile!\n\tError at pos %d: %s",
			lre->regexp,error_offset,error);
		return;
	}
	lre->re_extra = pcre_study(lre->re,0,&error);
	if(error!=NULL) {
		g_warning("Further study on /%s/ regexp did not succeed!\n\tError %s",
			lre->regexp,error);
		return;
	}
	g_message("Regular expression /%s/ compiled successfully.",lre->regexp);
}

//void match_re(LOGGER_REGEXP *lre, gchar *subject) {
void match_re(gpointer lrep, gpointer subjectp) {
	LOGGER_REGEXP *lre = (LOGGER_REGEXP *)lrep;
	gchar *subject = (gchar*) subjectp;
	gint ret;
	gint ovector[30];
	
	if(lre==NULL)
			return;
	ret = pcre_exec(lre->re,lre->re_extra, subject, strlen(subject),0,0,ovector,30);
	
	if(ret>=0) {
		//TODO matches!
		VteTerminal *vte;
		
		vte = get_terminal(lre->target_window);
		if(vte==NULL) {
//			g_message("Creating vte terminal for \"%s\"",lre->target_window);
			vte = add_term_page(lre->target_window);
		}
		vte_terminal_feed(vte, subject, strlen(subject));
	}
}

void study_all_re() {
	g_slist_foreach(logger_config.regexps, study_re, NULL);
}

void init_plugin(PLUGIN_OBJECT *plugin, gint context)
{
	gchar *fullpath;
	
	gladexml = NULL;
	parse_config();
	study_all_re();
	plugin_register_menu(context, "Logger Plugin", "show_preferences");
	
/*	fullpath = lookup_conf_file("logger.rc");
	if(fullpath == NULL) {
		g_warning("No \"logger.rc\" file found!");
	} else {
		g_message("Conf file \"logger.conf\" found in \"%s\"",fullpath);
		gtk_rc_parse(fullpath);
		g_free(fullpath);
	}*/
	if(logger_config.enabled) {
//		g_message("Enabled!");
		fullpath = lookup_conf_file("logger.glade");
		if(fullpath==NULL) {
			g_warning("No logger.glade file found!");
		} else {
			g_message("Conf file \"logger.glade\" found in \"%s\"",fullpath);
			gladexml = glade_xml_new(fullpath,"loggerwin",NULL);	
			g_free(fullpath);

			GSList *sl = logger_config.regexps;
			while (sl!=NULL) {
				if(sl->data != NULL)
					add_term_page(((LOGGER_REGEXP*)sl->data)->target_window);
				sl = g_slist_next(sl);
			}
		}
	}
	
	plugin_register_data_incoming(context, "data_in_function");
}

void data_in_function(PLUGIN_OBJECT *plugin, CONNECTION_DATA *connection, gchar *data, gint context)
{
	g_slist_foreach(logger_config.regexps,match_re,data);
}

void show_preferences(GtkWidget *widget, gint data)
{
	plugin_popup_message ("Not implemented yet!\nWait for next version!");
};
